@@ -1,7 +1,8 @@
 #ifndef BIOLCCC_H
 #define BIOLCCC_H
 
-#include <math.h>
+#include <cmath>
+#include "auxiliary.h"
 #include "biolcccexception.h"
 #include "chemicalbasis.h"
 #include "chromoconditions.h"
@@ -50,12 +51,18 @@ std::vector<ChemicalGroup> parseSequence(
     
     If \a continueGradient is true, than the last section of a gradient is
     prolonged.
+
+    If \a backwardCompatibility is true, than the calculated RT will be 
+    a multiple of dV/flow rate. This type of calculations was used in the 
+    version 1.1.0 and earlier.
 */
 double calculateRT(const std::string &sequence,
                    const ChemicalBasis & chemBasis,
                    const ChromoConditions & conditions =
                        standardChromoConditions,
-                   const bool continueGradient = true) 
+                   const int numInterpolationPoints = 0,
+                   const bool continueGradient = true,
+                   const bool backwardCompatibility = false) 
                    throw(BioLCCCException);
 
 //! Calculates the average (molar) mass of a peptide.
