@@ -345,21 +345,16 @@ double calculateKdChain(
         {
             for (unsigned int j = 0; j < latticeSize; j++)
             {
-                //std::cout << transitionMatrix[j + i * latticeSize] << " ";
                 densityBuffer[i] = densityBuffer[i] + density[j] *
                                    transitionMatrix[j + i * latticeSize];
             }
-            //std::cout << std::endl;
         }
 
         // Transferring the results from the density vector.
         for (unsigned int i = 0; i < latticeSize; i++)
         {
             density[i] = densityBuffer[i];
-            //std::cout << density[i] << " ";
         }
-        //std::cout << std::endl;
-        //std::cout << std::endl;
     }
 
     // Kd is a sum of elements of the density vector, normalized to the size 
@@ -598,12 +593,122 @@ double calculateKd(const std::vector<ChemicalGroup> &parsedSequence,
     }
 }
 
+class KdCalculator
+{
+public:
+    KdCalculator(const std::vector<ChemicalGroup> &parsedSequence,
+                 const ChemicalBasis &chemBasis, 
+                 const double columnPoreSize,
+                 const double columnRelativeStrength,
+                 const double temperature,
+                 const int numInterpolationPoints) :
+                     mParsedSequence(parsedSequence),
+                     mChemicalBasis(chemBasis),
+                     mColumnPoreSize(columnPoreSize),
+                     mColumnRelativeStrength(columnRelativeStrength),
+                     mTemperature(temperature),
+                     mN(numInterpolationPoints)
+    {
+        if (mN > 0)
+        {
+            mSecondSolventConcentrations = new double[mN];
+            mLogKds = new double[mN];
+            // The number of extra points in the terminal segments.
+            // This points significantly increase the accuracy of spline
+            // interpolation.
+            int NETP = 1;
+            for (int i=0; i < mN; i++) 
+            {
+                if (i <= NETP)
+                {
+                    mSecondSolventConcentrations[i] =
+                        i * 100.0 / (mN - 2.0 * NETP - 1.0) / (NETP + 1.0);
+                }
+                else if (i > (mN - NETP - 2))
+                {
+                    mSecondSolventConcentrations[i] = 
+                        ((mN - 2.0 * NETP - 2.0)
+                            + (i - mN + NETP + 2.0) / (NETP + 1.0))
+                         * 100.0 / (mN - 2.0 * NETP - 1.0);
+                }
+                else
+                {
+                    mSecondSolventConcentrations[i] =
+                        (i - NETP) * 100.0 / (mN - 2.0 * NETP - 1.0);
+                }
+                mLogKds[i] = log(calculateKd(mParsedSequence,
+                    mSecondSolventConcentrations[i],
+                    mChemicalBasis, 
+                    mColumnPoreSize,
+                    mColumnRelativeStrength,
+                    mTemperature));
+            }
+
+            mSecondDers = new double[mN];
+            fitSpline(mSecondSolventConcentrations, mLogKds,
+                mN, mSecondDers);
+        }
+    }
+
+    ~KdCalculator()
+    {
+        if (mN > 0)
+        {
+            delete[] mSecondSolventConcentrations;
+            delete[] mLogKds;
+            delete[] mSecondDers;
+        }
+    }
+
+    double operator()(double secondSolventConcentration) 
+        throw (BioLCCCException)
+    {
+        if (mN == 0) 
+        {
+            return calculateKd(mParsedSequence,
+                               secondSolventConcentration,
+                               mChemicalBasis, 
+                               mColumnPoreSize,
+                               mColumnRelativeStrength,
+                               mTemperature);
+        }
+        else 
+        {
+            return exp(calculateSpline(mSecondSolventConcentrations, mLogKds,
+                mSecondDers, mN, 
+                secondSolventConcentration));
+            //return exp(partPolInterpolate(
+            //    mSecondSolventConcentrations, mLogKds,
+            //    mN, 2,
+            //    secondSolventConcentration));
+        }
+    }
+
+private:
+    const std::vector<ChemicalGroup> &mParsedSequence;
+    const ChemicalBasis & mChemicalBasis;
+    const double mColumnPoreSize;
+    const double mColumnRelativeStrength;
+    const double mTemperature;
+    const int mN;
+    double * mSecondSolventConcentrations;
+    double * mLogKds;
+    double * mSecondDers;
+};
+
 double calculateRT(const std::vector<ChemicalGroup> &parsedSequence,
                    const ChemicalBasis &chemBasis,
                    const ChromoConditions &conditions,
-                   const bool continueGradient) throw(BioLCCCException)
+                   const int numInterpolationPoints,
+                   const bool continueGradient,
+                   const bool backwardCompatibility) throw(BioLCCCException)
 {
     // Calculating column volumes.
+    if (numInterpolationPoints < 0)
+    {
+        throw BioLCCCException(
+            "The number of interpolation points must be non-negative.");
+    }
     double volumeLiquidPhase = conditions.columnDiameter() *
                                conditions.columnDiameter() * 3.1415 * 
                                conditions.columnLength() / 4.0 /
@@ -651,25 +756,29 @@ double calculateRT(const std::vector<ChemicalGroup> &parsedSequence,
                 conditions.secondSolventConcentrationB()));
     }
 
-    // The part of a column passed by molecules. When it exceeds 1.0,
-    // molecule elute from the column.
-    double S = 0.0;
-    // The current iteration number.
-    int j = 0;
-    double secondSolventConcentration = 0.0;
-
     std::vector<std::pair<int, double> >::const_iterator currentGradientPoint=
         convertedGradient.begin();
     std::vector<std::pair<int, double> >::const_iterator previousGradientPoint=
         convertedGradient.begin();
+
+    KdCalculator kdCalculator(parsedSequence, chemBasis,
+                              conditions.columnPoreSize(),
+                              conditions.columnRelativeStrength(),
+                              conditions.temperature(),
+                              numInterpolationPoints);
+
+    double secondSolventConcentration = 0.0;
+    // The part of a column passed by molecules. When it exceeds 1.0,
+    // molecule elute from the column.
+    double S = 0.0;
+    double dS = 0.0;
+    // The current iteration number.
+    int j = 0;
     while (S < 1.0)
     {
         j++;
         if (j > currentGradientPoint->first)
         {
-            // If j exceeds the last point of a gradient, the value of the
-            // second solvent concentration is calculated by a prolongation of
-            // the last gradient section.
             if (currentGradientPoint != --convertedGradient.end())
             {
                 previousGradientPoint = currentGradientPoint;
@@ -681,66 +790,59 @@ double calculateRT(const std::vector<ChemicalGroup> &parsedSequence,
                         previousGradientPoint->second)
                 {
                     // One case is that a peptide elutes during this section or
-                    // the section is the last.
+                    // the section is the last. Then we can calculate the
+                    // retention time directly.
                     bool peptideElutes = 
-                        ((1.0 - S) / dV * calculateKd(
-                            parsedSequence,
-                            currentGradientPoint->second,
-                            chemBasis, 
-                            conditions.columnPoreSize(),
-                            conditions.columnRelativeStrength(),
-                            conditions.temperature()) * volumePore < 
+                        ((1.0 - S) / dV
+                            * kdCalculator(currentGradientPoint->second)
+                            * volumePore < 
                         (currentGradientPoint->first - j + 1));
 
                     if (peptideElutes ||
                         (currentGradientPoint == --convertedGradient.end()))
                     {
-                        j += (int)ceil((1.0 - S) / dV *
-                            calculateKd(
-                               parsedSequence,
-                               currentGradientPoint->second,
-                               chemBasis,
-                               conditions.columnPoreSize(),
-                               conditions.columnRelativeStrength(),
-                               conditions.temperature()) *
-                            volumePore);
+                        dS = dV / kdCalculator(currentGradientPoint->second) 
+                            / volumePore;
+                        j += (int) ceil((1.0 - S) / dS);
+                        S += (int) ceil((1.0 - S) / dS) * dS;
                         break;
                     }
+
                     // Another case is that this section is not long enough for
-                    // a peptide to elute.
+                    // a peptide to elute. In this case we calculate the
+                    // increase of S during this section.
                     else
                     {
-                        S += dV / calculateKd(
-                                 parsedSequence,
-                                 currentGradientPoint->second,
-                                 chemBasis,
-                                 conditions.columnPoreSize(),
-                                 conditions.columnRelativeStrength(),
-                                 conditions.temperature()) /
-                             volumePore * (currentGradientPoint->first -
-                                           previousGradientPoint->first);
+                        S += dV / kdCalculator(currentGradientPoint->second)
+                             / volumePore * (currentGradientPoint->first -
+                                             previousGradientPoint->first);
                         j = currentGradientPoint->first;
                     }
                 }
             }
+            // If j exceeds the last point of a gradient, the value of the
+            // second solvent concentration is calculated by a prolongation of
+            // the last gradient section.
             else if (!continueGradient)
             {
                 break;
             }
         }
+
         secondSolventConcentration = currentGradientPoint->second -
             (currentGradientPoint->second - previousGradientPoint->second) /
             (currentGradientPoint->first - previousGradientPoint->first) *
-            (currentGradientPoint->first - (double) j);
-        S += dV / calculateKd(
-                 parsedSequence, secondSolventConcentration,
-                 chemBasis, conditions.columnPoreSize(),
-                 conditions.columnRelativeStrength(),
-                 conditions.temperature()) / volumePore;
+            (currentGradientPoint->first - (double) (j-0.5));
+        dS = dV / kdCalculator(secondSolventConcentration) / volumePore;
+        S += dS;
     }
 
     double RT = j * dV / conditions.flowRate() + conditions.delayTime() +
                 volumeLiquidPhase / conditions.flowRate();
+    if (!backwardCompatibility)
+    {
+        RT -= (S - 1.0) / dS * dV / conditions.flowRate();
+    }
     return RT;
 }
 }
@@ -909,38 +1011,24 @@ std::vector<ChemicalGroup> parseSequence(
     parsedSequence.push_back(CTerminus);
     return parsedSequence;
 
-    /*
-    // Finally, we build an energy profile if it was defined.
-    if (peptideEnergyProfile != NULL)
-    {
-        peptideEnergyProfile->clear();
-        for (std::vector<ChemicalGroup>::const_iterator currentAminoAcid =
-                    parsedSequence.begin();
-                currentAminoAcid != parsedSequence.end();
-                currentAminoAcid++)
-        {
-            peptideEnergyProfile->push_back(currentAminoAcid->bindEnergy());
-        }
-
-        // Modifing energies of terminal amino acid residues.
-        *(peptideEnergyProfile->begin()) = *(peptideEnergyProfile->begin()) +
-                                           NTerminus->bindEnergy();
-        *(--peptideEnergyProfile->end()) = *(--peptideEnergyProfile->end()) +
-                                           CTerminus->bindEnergy();
-    }
-    */
 }
 
 double calculateRT(const std::string &sequence,
                    const ChemicalBasis &chemBasis,
                    const ChromoConditions &conditions,
-                   const bool continueGradient) 
+                   const int numInterpolationPoints,
+                   const bool continueGradient,
+                   const bool backwardCompatibility) 
                    throw(BioLCCCException)
 {
-    return calculateRT(parseSequence(sequence, chemBasis),
+    std::vector<ChemicalGroup> parsedSequence = 
+        parseSequence(sequence, chemBasis);
+    return calculateRT(parsedSequence,
                        chemBasis,
                        conditions,
-                       continueGradient);
+                       numInterpolationPoints,
+                       continueGradient,
+                       backwardCompatibility);
 }
 
 double calculateKd(const std::string &sequence,
